@@ -25,10 +25,10 @@ def main(logging_level: str = 'INFO', test: bool = False):
     """
     if 'ASTRALITY_LOGGING_LEVEL' in os.environ:
         # Override logging level if env variable is set
-        logger.setLevel(os.environ['ASTRALITY_LOGGING_LEVEL'])
-    else:
-        # Set the logging level to the level passed in by bin/astrality cli
-        logger.setLevel(logging_level)
+        logging_level = os.environ['ASTRALITY_LOGGING_LEVEL']
+
+    # Set the logging level to the configured setting
+    logging.basicConfig(level=logging_level)
 
     # Quit old astrality instances
     kill_old_astrality_processes()
@@ -72,14 +72,12 @@ def main(logging_level: str = 'INFO', test: bool = False):
         config = user_configuration()
 
         # Delay further actions if configuration says so
-        startup_delay = float(config.get('settings/general', {}).get('startup_delay', '0'))
+        startup_delay = float(config.get('settings/astrality', {}).get('startup_delay', 0))
         time.sleep(startup_delay)
 
         module_manager = ModuleManager(config)
         module_manager.finish_tasks()
 
-        # TODO: Implement startup delay config option
-
         while True:
             if module_manager.has_unfinished_tasks():
                 # TODO: Log which new period which has been detected
@@ -99,7 +97,7 @@ def main(logging_level: str = 'INFO', test: bool = False):
                     module_manager.time_until_next_period().total_seconds()
                 )
 
-    except KeyboardInterrupt:
+    except KeyboardInterrupt:  # pragma: no cover
         exit_handler()
 
 def other_astrality_pids() -> Set[int]:
