@@ -11,14 +11,14 @@ from astrality.config import generate_expanded_env_dict, user_configuration
 def conf_path():
     """Return str path to configuration directory."""
     this_test_file = os.path.abspath(__file__)
-    conf_path = Path(this_test_file).parents[2]
-    return str(conf_path)
+    conf_path = Path(this_test_file).parents[1] / 'config'
+    return conf_path
 
 
 @pytest.fixture
 def conf_file_path(conf_path):
     """Return path to example configuration."""
-    return os.path.join(conf_path, 'astrality.yaml.example')
+    return conf_path / 'astrality.yaml'
 
 
 @pytest.fixture(scope='session', autouse=True)
