@@ -3,11 +3,17 @@
 import logging
 import subprocess
 from pathlib import Path
+from typing import Union
 
 logger = logging.getLogger('astrality')
 
 
-def run_shell(command: str, working_directory: Path = Path.home()) -> str:
+def run_shell(
+    command: str,
+    timeout: Union[int, float] = 2,
+    fallback: str = '',
+    working_directory: Path = Path.home(),
+) -> str:
     """Return the standard output of a shell command."""
 
     process = subprocess.Popen(
@@ -19,7 +25,7 @@ def run_shell(command: str, working_directory: Path = Path.home()) -> str:
         stderr=subprocess.PIPE,
     )
     try:
-        process.wait(timeout=2)
+        process.wait(timeout=timeout)
 
         for error_line in process.stderr:
             logger.error(str(error_line))
@@ -29,7 +35,7 @@ def run_shell(command: str, working_directory: Path = Path.home()) -> str:
                 f'Command "{command}" exited with non-zero return code: ' +
                 str(process.returncode)
             )
-            return ''
+            return fallback
         else:
             stdout = process.communicate()[0]
             logger.info(stdout)
@@ -41,4 +47,4 @@ def run_shell(command: str, working_directory: Path = Path.home()) -> str:
             'finish. The exit code can not be verified. This might be '
             'intentional for background processes and daemons.'
         )
-        return ''
+        return fallback
